@@ -1,4 +1,4 @@
-package CTK::Util; # $Id: Util.pm 136 2013-03-16 15:47:57Z minus $
+package CTK::Util; # $Id: Util.pm 145 2013-09-03 10:27:19Z minus $
 use strict; # use Data::Dumper; $Data::Dumper::Deparse = 1;
 
 =head1 NAME
@@ -7,11 +7,11 @@ CTK::Util - CTK Utilities
 
 =head1 VERSION
 
-Version 2.11
+Version 2.12
 
 =head1 REVISION 
 
-$Revision: 136 $
+$Revision: 145 $
 
 =head1 SYNOPSIS
 
@@ -48,18 +48,24 @@ This function returns result of expression:
 
     time() - $^T
 
+Tags: BASE, DATE
+
 =head3 bload
 
     $bindata = bload( $file_or_fh, $onutf8 );
 
 Reading file in binary mode as ':raw:utf8' layer (if $onutf8 is true) or regular binary layer.
 
+Tags: BASE, FILE, ATOM
+
 =head3 bsave
 
     $status = bsave( $file_or_fh, $bindata, $onutf8 );
 
 Saving file in binary mode as ':raw:utf8' layer (if $onutf8 is true) or regular binary layer.
 
+Tags: BASE, FILE, ATOM
+
 =head3 catdir, catfile, rootdir, tmpdir, updir, curdir, path, splitpath, splitdir
 
 This is the L<File::Spec> functions, and exported here for historical reasons.
@@ -136,54 +142,72 @@ Returns a string representation of the parent directory.
 
 =back
 
+Tags: API, BASE
+
 =head3 cdata
 
     $cdatatext = cdata( $text );
 
 Returns a string "<![CDATA[$text]]>" for plain XML documents.
 
+Tags: BASE, FORMAT
+
 =head3 correct_date
 
     $mydate = correct_date( $date );
 
 Returns date in format dd.mm.yyyy or null ('') if $date is wrongly.
 
+Tags: BASE, DATE
+
 =head3 correct_dig
 
     $mydig = correct_dig( $string );
 
 Returns digits only from string or 0 if string is not correctly.
 
+Tags: BASE, FORMAT
+
 =head3 correct_number
 
     $mynumber = correct_number( $string, $sep );
 
 Placement of separators discharges among digits. For example 1`234`567 if $sep is char "`" (default)
 
+Tags: BASE, FORMAT
+
 =head3 current_date
 
     $date = current_date();
 
 Returns current date in format dd.mm.yyyy
 
+Tags: BASE, DATE
+
 =head3 current_date_time
 
     $datetime = current_date_time();
 
 Returns current date in format dd.mm.yyyy hh.mm.ss
 
+Tags: BASE, DATE
+
 =head3 date_time2dig
 
     $dtd = date_time2dig( $datetime );
 
 Returns $datetime (or current) in format yyyymmddhhmmss
 
+Tags: BASE, DATE
+
 =head3 date2dig
 
     $dd = date2dig( $date );
 
 Returns $date (or current) in format yyyymmdd
 
+Tags: BASE, DATE
+
 =head3 date2localtime
 
     $time = date2localtime( $date );
@@ -193,10 +217,14 @@ Returns time from date format dd.mm.yyyy in time() value in seconds since the sy
 
 See L<Time::Local/"timelocal">
 
+Tags: BASE, DATE
+
 =head3 datef
 
 See L</"dtf">
 
+Tags: BASE, DATE
+
 =head3 datetime2localtime
 
     $time = datetime2localtime( $datetime );
@@ -206,10 +234,14 @@ Returns time from datetime format dd.mm.yyyy hh.mm.ss in time() value in seconds
 
 See L<Time::Local/"timelocal">
 
+Tags: BASE, DATE
+
 =head3 datetimef
 
 See L</"dtf">
 
+Tags: BASE, DATE
+
 =head3 dformat
 
     $string = dformat( $mask, \%replacehash );
@@ -227,18 +259,24 @@ For example:
 
 See also L</"fformat"> for working with files
 
+Tags: BASE, FORMAT
+
 =head3 dig2date
 
     $date = dig2date_time( $dd );
 
 Returns date (or current) from format yyyymmdd in format dd.mm.yyyy
 
+Tags: BASE, DATE
+
 =head3 dig2date_time
 
     $datetime = dig2date_time( $dtd );
 
 Returns date (or current) from format yyyymmddhhmmss in format dd.mm.yyyy hh.mm.ss
 
+Tags: BASE, DATE
+
 =head3 dtf
 
     $datetime = dtf( $format, $time );
@@ -291,12 +329,16 @@ Examples:
 
 For more features please use L<Date::Format> and L<DateTime>
 
+Tags: BASE, DATE
+
 =head3 eqtime
 
     eqtime("source/file", "destination/file");
 
 Sets modified time of destination to that of source.
 
+Tags: BASE, FILE, ATOM
+
 =head3 escape
 
     $safe = escape("10% is enough\n");
@@ -307,6 +349,8 @@ be a string of bytes.
 
 See also L<URI::Escape>
 
+Tags: BASE, FORMAT
+
 =head3 fformat
 
     $file = fformat( $mask, $filename );
@@ -326,26 +370,36 @@ For example:
 
 See also L</"dformat">
 
+Tags: BASE, FORMAT
+
 =head3 file_load
 
 See L</"bload">
 
+Tags: BASE, FILE, ATOM
+
 =head3 file_save
 
 See L</"bsave">
 
+Tags: BASE, FILE, ATOM
+
 =head3 fload
 
     $textdata = fload( $file );
 
 Reading file in regular text mode
 
+Tags: BASE, FILE, ATOM
+
 =head3 fsave
 
     $status = fsave( $file, $textdata );
 
 Saving file in regular text mode
 
+Tags: BASE, FILE, ATOM
+
 =head3 getdirlist
 
     $listref = getdirlist( $dir, $mask );
@@ -354,6 +408,8 @@ Returns reference to array directories of directory $dir by $mask (regexp or sca
 
 See also L</"ls">
 
+Tags: BASE, FILE, ATOM
+
 =head3 getfilelist
 
     $listref = getlist( $dir, $mask );
@@ -362,26 +418,36 @@ Returns reference to array files of directory $dir by $mask (regexp or scalar st
 
 See also L</"ls">
 
+Tags: BASE, FILE, ATOM
+
 =head3 getlist
 
 See L</"getfilelist">
 
+Tags: BASE, FILE, ATOM
+
 =head3 load_file
 
 See L</"fload">
 
+Tags: BASE, FILE, ATOM
+
 =head3 localtime2date
 
     $date = localtime2date( time() )
 
 Returns time in format dd.mm.yyyy
 
+Tags: BASE, DATE
+
 =head3 localtime2date_time
 
     $datetime = localtime2date_time( time() )
 
 Returns time in format dd.mm.yyyy hh.mm.ss
 
+Tags: BASE, DATE
+
 =head3 ls
 
     @list = ls( $dir);
@@ -389,6 +455,8 @@ Returns time in format dd.mm.yyyy hh.mm.ss
 
 A function returns list content of directory $dir by $mask (regexp or scalar string)
 
+Tags: BASE, FILE, ATOM
+
 =head3 preparedir
 
     $status = preparedir( $dir );
@@ -403,6 +471,8 @@ The $chmode argument should be a octal value, for example:
 
     $status = preparedir( [qw/ foo bar baz /], 0777 );
 
+Tags: BASE, FILE, ATOM
+
 =head3 randchars
 
     $rand = randchars( $n ); # default chars collection: 0..9,'a'..'z','A'..'Z'
@@ -414,16 +484,22 @@ For example:
 
     $rand = randchars( 8, [qw/a b c d e f/]); # -> cdeccfdf
 
+Tags: BASE, FORMAT
+
 =head3 randomize
 
     $rand = randomize( $n );
 
 Returns random number of the set amount of characters
 
+Tags: BASE, FORMAT
+
 =head3 save_file
 
 See L</"fsave">
 
+Tags: BASE, FILE, ATOM
+
 =head3 scandirs
 
     @dirs = scandirs( $dir, $mask );
@@ -431,6 +507,8 @@ See L</"fsave">
 A function returns all directories of directory $dir by $mask (regexp or scalar string) in 
 format: [$path, $name]
 
+Tags: BASE, FILE, ATOM
+
 =head3 scanfiles
 
     @files = scanfiles( $dir, $mask );
@@ -438,6 +516,8 @@ format: [$path, $name]
 A function returns all files of directory $dir by $mask (regexp or scalar string) in 
 format: [$path, $name]
 
+Tags: BASE, FILE, ATOM
+
 =head3 shred
 
     $stat = shred( $file );
@@ -445,6 +525,8 @@ format: [$path, $name]
 Do a more secure overwrite of given files or devices, to make it harder for even very 
 expensive hardware probing to recover the data.
 
+Tags: BASE, FILE, ATOM
+
 =head3 shuffle
 
     @cards = shuffle(0..51); # 0..51 in a random order
@@ -456,21 +538,29 @@ Pure-Perl implementation of Function List::Util::PP::shuffle
 
 See also L<List::Util>
 
+Tags: BASE, FORMAT
+
 =head3 slash
 
     $slashed = slash( $string );
 
 Escaping symbols \ and ' and returns strings \\ and \'
 
+Tags: BASE, FORMAT
+
+=head3 splitformat
+
+See L</"fformat">
+
+Tags: BASE, FORMAT
+
 =head3 tag
 
     $detagged = tag( $string );
 
 <, >, " and ' chars convert to &lt;, &gt;, &quot; and &#39; strings.
 
-=head3 splitformat
-
-See L</"fformat">
+Tags: BASE, FORMAT
 
 =head3 tag_create
 
@@ -478,10 +568,14 @@ See L</"fformat">
 
 Reverse function L</"tag">
 
+Tags: BASE, FORMAT
+
 =head3 timef
 
 See L</"dtf">
 
+Tags: BASE, DATE
+
 =head3 to_base64
 
     $base64_text = to_base64( $utf8_text );
@@ -495,10 +589,14 @@ enabling 6 bits to be represented per printable character.
 
 See also L<MIME::Base64>
 
+Tags: BASE, FORMAT
+
 =head3 to_cp1251
 
 See L</"to_windows1251">
 
+Tags: BASE, FORMAT
+
 =head3 to_utf8
 
     $utf8_text = to_utf8( $win1251_text )
@@ -511,6 +609,8 @@ and aliases, see L<Encode>.
 
 Obsolete alias: B<CP1251toUTF8>
 
+Tags: BASE, FORMAT
+
 =head3 to_windows1251
 
     $win1251_text = to_windows1251( $utf8_text )
@@ -522,18 +622,24 @@ an alias. For encoding names and aliases, see L<Encode>.
 
 Obsolete alias: B<UTF8toCP1251>
 
+Tags: BASE, FORMAT
+
 =head3 touch
 
     touch( "file" );
 
 Makes file exist, with current timestamp
 
+Tags: BASE, FILE, ATOM
+
 =head3 translate
 
     $string = translate( $rus_string );
 
 Translation russian (windows-1251/CP-1251) chars in latin symbols (poland transcription)
 
+Tags: BASE, FORMAT
+
 =head3 unescape
 
     $str = unescape(escape("10% is enough\n"));
@@ -546,6 +652,8 @@ This does the same as:
 
 See also L<URI::Escape>
 
+Tags: BASE, FORMAT
+
 =head3 variant_stf
 
     $fixlenstr = variant_stf( "qwertyuiop", 3 ); # -> q.p
@@ -553,22 +661,30 @@ See also L<URI::Escape>
 
 Returns a line the fixed length from 3 to the n chars
 
+Tags: BASE, FORMAT
+
 =head3 visokos
 
     $lybool = visokos( 2012 );
 
 Returns a leap-year or not
 
+Tags: BASE, DATE
+
 =head2 UTILITY SUBROUTINES
 
 =head3 com
 
 See L</"execute">
 
+Tags: UTIL, ATOM
+
 =head3 exe
 
 See L</"execute">
 
+Tags: UTIL, ATOM
+
 =head3 execute
 
     $out = execute( "ls -la" );
@@ -583,6 +699,8 @@ regular binary layer (if $binmode is true).
 
 See also L<IPC::Open3>
 
+Tags: UTIL, ATOM
+
 =head3 ftp
 
     %ftpct = (
@@ -606,6 +724,8 @@ Simple working with FTP.
 
 See also L<Net::FTP>
 
+Tags: UTIL, ATOM
+
 =head3 ftpgetlist
 
     $rfiles = ftpgetlist( \%ftpct, $mask);
@@ -614,6 +734,8 @@ Returns reference to array of remote source listing by mask (as regexp, optional
 
 See L</"ftp">
 
+Tags: UTIL, ATOM
+
 =head3 ftptest
 
     $status = ftptest( \%ftpct );
@@ -622,6 +744,8 @@ FTP connect testing.
 
 See L</"ftp">
 
+Tags: UTIL, ATOM
+
 =head3 sendmail
 
     my $sent = sendmail(
@@ -669,10 +793,14 @@ See L</"ftp">
 
 Send mail. See L<MIME::Lite> for details
 
+Tags: UTIL, ATOM
+
 =head3 send_mail
 
 See L</"sendmail">
 
+Tags: UTIL, ATOM
+
 =head2 EXTENDED SUBROUTINES
 
 =head3 cachedir
@@ -693,6 +821,8 @@ a disk space shortage). No other requirements are made on the data format of the
 
 See L<http://www.pathname.com/fhs/pub/> and L<Sys::Path/"cachedir">
 
+Tags: API, BASE
+
 =head3 docdir
 
     my $value = docdir();
@@ -702,6 +832,8 @@ For example value can be set as: /usr/share/doc
 
 See L<Sys::Path/"docdir">
 
+Tags: API, BASE
+
 =head3 localedir
 
     my $value = localedir();
@@ -711,6 +843,8 @@ For example value can be set as: /usr/share/locale
 
 See L<Sys::Path/"localedir">
 
+Tags: API, BASE
+
 =head3 localstatedir
 
     my $value = localstatedir();
@@ -738,6 +872,8 @@ they have some system-wide implication, and in consultation with the FHS mailing
 
 See L<http://www.pathname.com/fhs/pub/> and L<Sys::Path/"localstatedir">
 
+Tags: API, BASE
+
 =head3 lockdir
 
     my $value = lockdir();
@@ -758,6 +894,8 @@ one, two, three, zero, and newline.
 
 See L<http://www.pathname.com/fhs/pub/> and L<Sys::Path/"lockdir">
 
+Tags: API, BASE
+
 =head3 prefixdir
 
     my $value = prefixdir();
@@ -777,6 +915,8 @@ Large software packages must not use a direct subdirectory under the /usr hierar
 
 See L<http://www.pathname.com/fhs/pub/> and L<Sys::Path/"prefix">
 
+Tags: API, BASE
+
 =head3 rundir
 
     my $value = rundir();
@@ -793,6 +933,8 @@ convention for PID files is <program-name>.pid. For example, the crond PID file
 
 See L<http://www.pathname.com/fhs/pub/> and L<Sys::Path/"rundir">
 
+Tags: API, BASE
+
 =head3 sharedir
 
     my $value = sharedir();
@@ -815,6 +957,8 @@ game play logs, and so forth, should be placed in /var/games.
 
 See L<http://www.pathname.com/fhs/pub/> and L<Sys::Path/"datadir">
 
+Tags: API, BASE
+
 =head3 sharedstatedir
 
     my $value = sharedstatedir();
@@ -839,6 +983,8 @@ distributions may use different names, of course.
 
 See L<http://www.pathname.com/fhs/pub/> and L<Sys::Path/"sharedstatedir">
 
+Tags: API, BASE
+
 =head3 spooldir
 
     my $value = spooldir();
@@ -852,6 +998,8 @@ processed.
 
 See L<http://www.pathname.com/fhs/pub/> and L<Sys::Path/"spooldir">
 
+Tags: API, BASE
+
 =head3 srvdir
 
     my $value = srvdir();
@@ -863,6 +1011,8 @@ For example value can be set as: /srv
 
 See L<http://www.pathname.com/fhs/pub/> and L<Sys::Path/"srvdir">
 
+Tags: API, BASE
+
 =head3 sysconfdir
 
     my $value = sysconfdir();
@@ -875,6 +1025,8 @@ of a program; it must be static and cannot be an executable binary.
 
 See L<http://www.pathname.com/fhs/pub/> and L<Sys::Path/"sysconfdir">
 
+Tags: API, BASE
+
 =head3 syslogdir
 
     my $value = syslogdir();
@@ -887,6 +1039,8 @@ subdirectory.
 
 See L<http://www.pathname.com/fhs/pub/> and L<Sys::Path/"logdir">
 
+Tags: API, BASE
+
 =head3 webdir
 
     my $value = webdir();
@@ -898,6 +1052,8 @@ Directory where distribution put static web files.
 
 See L<Sys::Path/"webdir">
 
+Tags: API, BASE
+
 =head2 CORE SUBROUTINES
 
 =head3 carp, croak, cluck, confess
@@ -932,10 +1088,14 @@ Die of errors with stack backtrace
 
 =back
 
+Tags: API, BASE
+
 =head3 getsyscfg
 
 See L</"syscfg">
 
+Tags: API, BASE
+
 =head3 isos
 
 Returns true or false if the OS name is of the current value of C<$^O>
@@ -946,6 +1106,8 @@ Returns true or false if the OS name is of the current value of C<$^O>
 
 See L<Perl::OSType> for details
 
+Tags: API, BASE
+
 =head3 isostype
 
 Given an OS type and OS name, returns true or false if the OS name is of the
@@ -959,6 +1121,8 @@ given type.
 
 See L<Perl::OSType/"is_os_type">
 
+Tags: API, BASE
+
 =head3 read_attributes
 
 Smart rearrangement of parameters to allow named parameter calling.
@@ -981,6 +1145,8 @@ We do the rearrangement if the first parameter begins with a -
 
 See L<CGI::Util>
 
+Tags: API, BASE
+
 =head3 syscfg
 
 Returns all hash %Config from system module L<Config> or one value of this hash
@@ -993,6 +1159,8 @@ Returns all hash %Config from system module L<Config> or one value of this hash
 
 See L<Config> module for details
 
+Tags: API, BASE
+
 =head2 TAGS
 
 =head3 ALL, DEFAULT
@@ -1301,8 +1469,8 @@ L</"preparedir">,
 L</"save_file">,
 L</"scandirs">,
 L</"scanfiles">,
-C</"send_mail">,
-C</"sendmail">,
+L</"send_mail">,
+L</"sendmail">,
 L</"shred">,
 L</"touch">
 
@@ -1383,7 +1551,7 @@ use constant {
 };
 
 use vars qw/$VERSION/;
-$VERSION = q/$Revision: 136 $/ =~ /(\d+\.?\d*)/ ? sprintf("%.2f",($1+100)/100) : '1.00';
+$VERSION = q/$Revision: 145 $/ =~ /(\d+\.?\d*)/ ? sprintf("%.2f",($1+100)/100) : '1.00';
 
 use Time::Local;
 use File::Spec::Functions qw/
@@ -2304,11 +2472,19 @@ sub ftptest {
 }
 sub ftpgetlist {
     #        
-    my $connect  = shift || {};  #  
-    my $mask     = shift || qr//; #  
+    my $connect  = shift || {}; #  
+    my $mask     = shift || ''; #  
     
     my $rfile = ftp($connect, 'ls'); 
-    my @files = grep {$_ =~ $mask} ($rfile ? @$rfile : ());
+    my @files = (($rfile && ref($rfile) eq 'ARRAY') ? @$rfile : ());
+
+    #      !
+    if ($mask && ref($mask) eq 'Regexp') {
+        @files = grep {$_ =~ $mask} @files; 
+    } else {
+        @files = grep {/$mask/} @files if $mask;
+    }
+
     return [@files];
 }
 sub execute {
